/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif

// this header
#include "VEML.hh"


using namespace PIAVE;
using std::string;
using std::cout;
using std::cerr;
using std::endl;

bool 
xmlCharHasContent( xmlChar* text )
{
    if ( !text ) return false;
    for ( char * c = (char*)text;
	  *c != '\0';
	  c++ )
	{
	    switch ( *c ) {
	    case ' ': break;
	    case '\x9': break;
	    case '\xa': break;
	    default :
		return true;
	    }
	}
    return false;
}


VEMLCmndMap  * VEMLCmnd::_allVEMLCommands = 0;

VEMLCmnd::VEMLCmnd( const std::string & n, VEMLCall c, VEMLCmndType t ) : 
  PropertyNode( n ),
  _type( t ),
  _call( c )
{
}

VEMLCmnd::VEMLCmnd( const VEMLCmnd & o ) :
  PropertyNode( o ),
  _type( o._type ),
  _call( o._call )
{
}

void
VEMLCmnd::copyFromXMLNode( const xmlDocPtr doc, const xmlNodePtr cmndNode )
{
    /*  
     *     copy attributes
     */

    for( xmlAttr* attr = cmndNode->properties; 
         attr; 
         attr = attr->next )
    {
        // FIXME ... this looks clumsy
        xmlChar* xmlValue = xmlGetProp( cmndNode, attr->name);
        string strValue = xmlValue ? (char*)xmlValue : "";
        xmlFree( xmlValue );
        addAttr( (const char*)attr->name, strValue );
    }

    /*
     *    copy children
     */
    for ( xmlNodePtr xmlCmndNode = cmndNode->xmlChildrenNode;
          xmlCmndNode != 0;
          xmlCmndNode = xmlCmndNode->next )
    {
        if ( !xmlStrcmp(xmlCmndNode->name, (const xmlChar *)"text") ) {
	    xmlChar * xmlContent = xmlNodeListGetString( doc, xmlCmndNode, 1 );
	    if ( xmlCharHasContent( xmlContent ) ) {
		string content( (char *)xmlContent );
		setContent( content );
	    }
	    xmlFree( xmlContent );
	} else {
	    VEMLCmnd * cmnd = new VEMLCmnd( (const char*)xmlCmndNode->name );
	    cmnd->copyFromXMLNode( doc, xmlCmndNode );
	    addDaughter( cmnd );
	}
    }

    /*
     *    copy content
     */
//     for ( xmlNodePtr xmlCmndNode = cmndNode->xmlChildrenNode;
//           xmlCmndNode != 0;
//           xmlCmndNode = xmlCmndNode->next )
//     {
// 	if ( !xmlStrcmp(xmlCmndNode->name, (const xmlChar *)"text") ) {
//     }


}

// void
// VEMLCmnd::print( std::ostream & o, int indent ) const
// {
//     string s( indent, ' ' );
//     o << s <<_name << endl 
//       << s << "  type: ";
//     switch ( _type ) {
//       case stdQ:     o<<"standard queue"; break;
//       case dropQ:    o<<"drop queue"; break;
//       case priority: o<<"priority"; break;
//       default:       o<<"unknown";
//     }
//     o << endl 
//       << s << "  attributes: ";
//     if ( _attrList ) {
//         o<<_attrList->size() << endl;
//         VEMLAttrList::const_iterator iter;
//         for ( iter =  _attrList->begin();
//               iter != _attrList->end();
//               ++iter )
//         {
//             o << s << "    Name: " << iter->first
//               << "  val: " << iter->second << endl;
//         }
//     } else {
//         o<<"none"<<endl;
//     }

//     if ( !_daughters ) return;
    
//     VEMLCmndList::const_iterator cmndIter;
//     for ( cmndIter =  _daughters->begin();
//           cmndIter != _daughters->end();
//           ++cmndIter )
//     {
//         (*cmndIter)->print( o, indent + 3 );
//     }

//     return;
// }


VEMLCmnd::~VEMLCmnd()
{
}

void
VEMLCmnd::operator=( const VEMLCmnd & o ) 
{
    PropertyNode::operator=( o );
    _type = o._type;
    _call = o._call;
//     PDELETE( _attrList );
//     if ( o._attrList ) {
//         _attrList = new VEMLAttrList( *o._attrList );
//     }

    // daughter commands are not copied

    return;
}

