/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#if ! defined ( VEML_HH )
#define VEML_HH

// std
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>

// std++
#include <iostream>
#include <string>
#include <algorithm>
#include <functional>
#include <list>

// piave
#include "piave_base.hh"
#include "storybase.hh"
#include "property.hh"


// libxml
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>


namespace PIAVE {

    /** Video Editing Markup Language ;-) */
    class VEMLCmnd;
    class KdenLiveServer;

    typedef std::vector< VEMLCmnd* > VEMLCmndList;
    typedef std::map< const std::string, VEMLCmnd > VEMLCmndMap;

    typedef std::map< std::string, std::string > VEMLAttrList;


    typedef PropertyNode * (KdenLiveServer::*VEMLCall)( const VEMLCmnd * );

    class VEMLCmnd : public PropertyNode
    {
      public:
        enum VEMLCmndType { stdQ, dropQ, priority };
        VEMLCmnd( const std::string & n = std::string(""), 
                  VEMLCall c=0, VEMLCmndType t = stdQ );
        VEMLCmnd( const VEMLCmnd & o );
        ~VEMLCmnd();

        void copyFromXMLNode( const xmlDocPtr doc, const xmlNodePtr o );

        VEMLCmndType   type()      const { return _type; }
        VEMLCall       call()      const { return _call; }

        void setType( VEMLCmndType t ) { _type = t; }
        void setCall( VEMLCall c ) { _call = c; }

        static void setAllCmnds( VEMLCmndMap * c ) { _allVEMLCommands = c; }

        bool operator==( const VEMLCmnd & o ) const {
            return getName() == o.getName();
        }
        void operator=( const VEMLCmnd & o );

      private:
        VEMLCmndType        _type;
        VEMLCall            _call;
        static VEMLCmndMap       * _allVEMLCommands;
    };


}

#endif
