/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** 
 * \file frame.cc implements frame.hh
 *
 * based on:
 *
 * frame.cc -- utilities for process digital video frames
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif

// this header
#include "algorithm.hh"

// PIAVE
#include "libpiave/piave_base.hh"

using namespace PIAVE;

/*
 *   Algorithm_C
 *
 */

/* RGB2YCbCr lookup tables */
int32_t Algorithm_C::RGBYCbCr02570[256];
int32_t Algorithm_C::RGBYCbCr05040[256];
int32_t Algorithm_C::RGBYCbCr00980[256];
int32_t Algorithm_C::RGBYCbCr01480[256]; 
int32_t Algorithm_C::RGBYCbCr02910[256];
int32_t Algorithm_C::RGBYCbCr04390[256];
int32_t Algorithm_C::RGBYCbCr03680[256];
int32_t Algorithm_C::RGBYCbCr00710[256];

Algorithm_C::Algorithm_C()
{
    init_RGB2YCbCr();
}

/* init lookup */
void 
Algorithm_C::init_RGB2YCbCr() 
{
    int i;

    for (i = 0; i < 256; i++)
	RGBYCbCr02570[i] = (int32_t) (0.2570 * i * 65536);
    
    for (i = 0; i < 256; i++)
	RGBYCbCr05040[i] = (int32_t) (0.5040 * i * 65536);
    
    for (i = 0; i < 256; i++)
	RGBYCbCr00980[i] = (int32_t) (0.0980 * i * 65536);

    for (i = 0; i < 256; i++)
	RGBYCbCr01480[i] = -(int32_t) (0.1480 * i * 65536);
    
    for (i = 0; i < 256; i++)
	RGBYCbCr02910[i] = -(int32_t) (0.2910 * i * 65536);

    for (i = 0; i < 256; i++)
	RGBYCbCr04390[i] = (int32_t) (0.4390 * i * 65536);

    for (i = 0; i < 256; i++)
	RGBYCbCr03680[i] = -(int32_t) (0.3680 * i * 65536);
    
    for (i = 0; i < 256; i++)
	RGBYCbCr00710[i] = -(int32_t) (0.0710 * i * 65536);

}

/*
 *     color spaces
 */

Color
Algorithm_C::RGB2YCbCr( Color rgb ) const
{
    Color yuv;

    yuv[0] = (uint8_t)( ( RGBYCbCr02570[ rgb[2] ] +
                          RGBYCbCr05040[ rgb[1] ] +
                          RGBYCbCr00980[ rgb[0] ] + 
                          0x100000  ) 
                        >> 16 );
    yuv[1] = (uint8_t)( ( RGBYCbCr01480[ rgb[2] ] +
                          RGBYCbCr02910[ rgb[1] ] +
                          RGBYCbCr04390[ rgb[0] ] + 
                          0x800000  ) 
                        >> 16 );
		    
    yuv[2] = (uint8_t)( ( RGBYCbCr04390[ rgb[2] ] +
                          RGBYCbCr03680[ rgb[1] ] +
                          RGBYCbCr00710[ rgb[0] ] + 
                          0x800000  ) 
                        >> 16 );

    yuv[3] = rgb[3]; // alpha

    return yuv;
}




/*
 *     Algos on frames FRAMES
 */


int 
Algorithm_C::yuyv2yuyv( int x_src,  int y_src,  uint8_t * src,
                        int x_dest, int y_dest, uint8_t * dest) const 
{
    ABORT( "FIXME" );
}

int 
Algorithm_C::rgb242yuyv( int x_src,  int y_src,  uint8_t * src,
                         int x_dest, int y_dest, uint8_t * dest) const 
{
    ABORT( "FIXME" );
}
