/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** 
 * \file frame.hh
 * Code for handling video and audio frames
 *
 * Classes in this file represent audio and video frames and all operations 
 * related to allocation and conversion of raw data buffers.
 *
 *
 * based on:
 *
 * frame.h -- utilities for process digital video frames
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
 */

#if ! defined ( ALGORITHM_HH )
#define ALGORITHM_HH

// std c
#include <stdint.h>

// std c++

// PIAVE

namespace PIAVE {

    class Color;

    /*   \class Algorithm this is a collection of usefull algorithms
     *   for video scaling and colorspace conversion Algorithm defines
     *   the interface which has a reference implementation in
     *   C(++). At some point there should be other implementations
     *   for specific architectures like MMX 3DNOW, SSE, SSE2,
     *   AltiVec, you name it.  Those can inherit Algorithm_C if they
     *   don't implement all functions.
     *
     *   It is also possible to mplement _fast and/or _good versions
     *   of the algoritms.
     */

    class Algorithm 
    {
      public:
        /*
         *     color spaces
         */
        virtual Color RGB2YCbCr( Color rgb ) const = 0;

        /*
         *     Algos on frames FRAMES
         */
        virtual int yuyv2yuyv( int x_src,  int y_src,  uint8_t * src,
                               int x_dest, int y_dest, uint8_t * dest) const = 0;
        virtual int rgb242yuyv( int x_src,  int y_src,  uint8_t * src,
                                int x_dest, int y_dest, uint8_t * dest) const = 0;

    };


    class Algorithm_C : public Algorithm
    {
      public:
        Algorithm_C();

        virtual Color RGB2YCbCr( Color rgb ) const;

        virtual int yuyv2yuyv( int x_src,  int y_src,  uint8_t * src,
                               int x_dest, int y_dest, uint8_t * dest) const;
        virtual int rgb242yuyv( int x_src,  int y_src,  uint8_t * src,
                                int x_dest, int y_dest, uint8_t * dest) const;

      private:
        void init_RGB2YCbCr();

        static int32_t RGBYCbCr02570[256], RGBYCbCr05040[256], RGBYCbCr00980[256];
        static int32_t RGBYCbCr01480[256], RGBYCbCr02910[256], RGBYCbCr04390[256];
        static int32_t RGBYCbCr03680[256], RGBYCbCr00710[256];

    };


};

#endif 
