/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file player.hh implements a simple main loop and all infrastructure to try to
 * run this main loop in the requested speed.
 */

#if ! defined( AVCCONTROLLER_HH )
#define AVCCONTROLLER_HH

// stdc
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/time.h>

// stdc++
#include <string>

// PIAVE
#include "piave_base.hh"
#include "storybase.hh"

// IEEE1394
#include <libraw1394/raw1394.h>
#include <libraw1394/csr.h>
#include "dv1394.h"

namespace PIAVE {

    /*
     *  \class AVCController
     */

    class AVCController
    {
      public:
        enum ControllerState { 
            uninitialized, 
            error, 
            not_connected, 
            connected 
        };
        
      public:
        static AVCController * inst() { 
            if (!_singleton) _singleton = new AVCController();
            return _singleton; 
        }

        ControllerState  getState() const { return _controllerState; }
        int              getPort()   const { return _port; }
        int              getNPorts() const { return _nports; }
        quadlet_t        getCamState();
        std::string      getCamStateName();
        std::string      getTimeCode();
        raw1394handle_t  getRaw1394Handle() const { return _raw1394_handle; }

        void             setControllerState( ControllerState state ) { _controllerState = state; }
        void             setPort( int port ) { _port = port; }

        int              findAVCNode( int phyID = 0 );

        void             init();

        void             vcrStop();
        void             vcrPause();
        void             vcrPlay();
        void             vcrPlaySlow();
        void             vcrReverse();
        void             vcrReverseSlow();
        void             vcrTrickPlay( int speed );
        void             vcrRewind();
        void             vcrForward();
        void             vcrNext();
        void             vcrPrevious();

        void             vcrSeekTimeCode( std::string timecode );
        
      private:
        AVCController();
        ~AVCController(){};

        static int reset_handler( raw1394handle_t handle, unsigned int generation );

      private:
        ControllerState     _controllerState;

        raw1394handle_t     _raw1394_handle;
        int                 _nports;
        int                 _port;
        int                 _avc_node;

        static AVCController * _singleton;

    };


}

#endif
