/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file avstream.cc implements avstream.hh
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif



// std++
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <string>
using std::string;

// this header
#include "avstream.hh"
#include "property.hh"

// PIAVE
#include "frame.hh"

using namespace PIAVE;

InAVStreamIFace::InAVStreamIFace() : 
  Stream(), 
   _deliveredFrames( 0 ), 
  _frameProperties( new PropertyNode("FrameProperties") ) 
{
}

InAVStreamIFace::~InAVStreamIFace()
{
    PDELETE( _frameProperties );
    INFO( PRINTV( _deliveredFrames ) );
}


OutAVStreamIFace::~OutAVStreamIFace()
{
    INFO( PRINTV( _recievedFrames ) );
}


/***
 *
 *  class FileStream 
 *
 */
InFileStream::InFileStream( const string& fileName )
  : _fd( -1 ),
    _fileName( fileName ),
//    _ifstream( fileName ),
    _length( 0 ),
    _pos( 0 )
{
//    WARN_IF( !_ifstream , "bad InFileStream filename="<<fileName );
    _fd = open( fileName.c_str(), O_RDONLY | O_NONBLOCK );
    // FXME : handle this gracefully
    ABORT_IF ( _fd < 0 ,  "InFileStream::InFileStream : ERROR opening \""<<fileName<<"\"" );
    fstat( _fd, &_stat );
}

InFileStream::~InFileStream()
{
    close( _fd );
}

off_t 
InFileStream::getFileSize()
{
    return _stat.st_size;
}


/***
 *
 *  class OutFileStream
 *
 */
OutFileStream::OutFileStream( const string& fileName )
  : _fd( -1 ),
    _fileName( fileName )
//    _ofstream( fileName )
{
//    WARN_IF( !_ofstream , "bad InFileStream filename="<<fileName );
    _fd = open( fileName.c_str(), O_CREAT | O_TRUNC | O_RDWR, 0644);
    if ( _fd == -1 ) {
        WARN( "error opening file \""<<fileName<<"\"" );
        throw file_error( "cannot open file" );
    }
}

OutFileStream::~OutFileStream()
{
    closeFD();
}


void
OutFileStream::closeFD()
{
    if ( _fd>0 ) {
        INFO( "closing" );
        close( _fd );
    } else {
        INFO( "wasn't open" );
    }
}
