/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** 
 * \file frame.cc implements frame.hh
 *
 * based on:
 *
 * frame.cc -- utilities for process digital video frames
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif



// C includes

// std
#include <string>
using std::string;
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

// this header
#include "bufferpool.hh"

// PIAVE
#include "piave_base.hh"

using namespace PIAVE;

/**
 *
 * \class BufferPool
 *
 * central facility to manage buffers for dv, video, and audio frames
 * this means: as many buffers as necessary, as few as possible.
 *
 */

BufferPool * BufferPool::_instance = 0;

BufferPool::BufferPool() 
{
}

BufferPool::~BufferPool()
{
    PoolMap::iterator iter;
    for( iter = _poolMap.begin();
         iter != _poolMap.end();
         ++iter )
    {
	    PDELETE(iter->second);
    }
}

void
BufferPool::printStats()
{
    PoolMap::iterator iter;
    for ( iter = BufferPool::Inst()->_poolMap.begin();
          iter != BufferPool::Inst()->_poolMap.end();
          ++iter )
    {
        int max_used = 0;
        int in_use = 0;
        for ( int i = 0;
              i < FBUFSIZE;
              i++ )
        {
              if ( iter->second->pool[i].buf() != 0 ) 
              {
                  max_used++;
                  if ( iter->second->pool[i].getRefCount() > 0 ) in_use++;
              }
        }
//        INFO( "Buffer size: "<<iter->first<<" in use: "<<in_use<<" max used: "<<max_used );
    }
}

// void
// BufferPool::clearAll()
// {
//     PoolMap::iterator iter;
//     for ( iter = BufferPool::Inst()->_poolMap.begin();
//           iter != BufferPool::Inst()->_poolMap.end();
//           ++iter )
//     {
//         iter->second->clear();
//     }
//     return;
// }


Buffer *
BufferPool::getNewBuffer( size_t size )
{
    Pool * pool = 0;

    /* do we already have a pool with the requested size?  */
    PoolMap::iterator iter = BufferPool::Inst()->_poolMap.find( size );

    if ( iter == BufferPool::Inst()->_poolMap.end() ) {
        /* no -> create it  */
        pool = new Pool( size );
        BufferPool::Inst()->_poolMap[ size ] = pool;
        printStats();
    } else {
        pool = iter->second;
    }

    Buffer * b = 0;
    int i;
    for ( i=0; i<FBUFSIZE && b==0; i++ )
    {
        if ( pool->pool[i].getRefCount() == 0 ) {
            b = &(pool->pool[i]);
        }
    }

    ABORT_IF( b==0, "BUFFEROVERFLOW" );

    if ( b->buf() == 0 ) {
        b->setBuffer( new uint8_t[ size ] );
    }

    b->incRef();

    return b;
}
