/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** \file
 * Here lives the server functionality of piave.
 *
 * This functionality is at the moment only used by kdenlive.  
 *
 * "Server" contains all functionality that would in principle be common
 * between serves for other GUIs, clients.
 *
 * "KdenLiveServer" implements the xml based protokoll to talk to kdenlive
 *
 */

#if ! defined ( EXCEPTION_HH )
#define EXCEPTION_HH

#include <exception>
#include <string>

namespace PIAVE {

    class exception: public std::exception
    {
      public:
        explicit exception( const std::string& message ) :
          _message( message ) {}
        virtual ~exception() throw() {};

        virtual const char* what() const throw() { return _message.c_str(); }

      private:
        std::string _message;
    };

    class stream_error : public exception
    {
      public:
        explicit stream_error(const std::string& message) : 
          exception( message ) {};
        virtual ~stream_error() throw() {};
    };

    class file_error : public stream_error
    {
      public:
        explicit file_error(const std::string& message) : 
          stream_error( message ) {};
        virtual ~file_error() throw() {};
    };

    class file_type_error : public file_error
    {
      public:
        explicit file_type_error(const std::string& message) : 
          file_error( message ) {};
        virtual ~file_type_error() throw() {};
    };

    class video_stream_error : public stream_error
    {
      public:
        explicit video_stream_error(const std::string& message) : 
          stream_error( message ) {};
        virtual ~video_stream_error() throw() {};
    };

    class audio_stream_error : public stream_error
    {
      public:
        explicit audio_stream_error(const std::string& message) : 
          stream_error( message ) {};
        virtual ~audio_stream_error() throw() {};
    };
    
    class codec_error : public stream_error
    {
      public:
        explicit codec_error(const std::string& message) :
          stream_error( message ) {};
        virtual ~codec_error() throw() {};
    };
    
} 

#endif
