/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file mediaelements.cc  implements mediaelements.hh
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif


// STL
#include <algorithm>

// this header
#include "mediaelements.hh"
#include "algorithm.hh"

// piave
using namespace PIAVE;
using std::string;
using std::cout;
using std::endl;

/**
 *  class AVChunk
 */
 
AVChunk::AVChunk( InAVFStream * s )
  : MediaElement( s?s->getFileName():"AVChunk", 0, s?s->length():0 ),
    _stream( s ), 
    _avfStreamOffset( 0 )
{
    INFO( "construct AVChunk with length="<<length()<<" name="<<name() );
}

AVChunk::AVChunk( const string& name, InAVFStream * s )
  : MediaElement( name, 0, s?s->length():0 ),
    _stream( s )
{
}

AVChunk::~AVChunk()
{
    PDELETE( _stream );
}

Frame 
AVChunk::getFrame( Time t )
{
    Time localTime = t - start();

    if ( !_stream ) {
        WARN( "no stream!{ " );
        return Frame();
    }

    if ( localTime>length() ) {
        WARN( "Trying to read beyond end of chunk "<<PRINTV( localTime )<<PRINTV( length() ) );
        return  _stream->getFrame( _avfStreamOffset + length() - 1 );
    }

    return _stream->getFrame( _avfStreamOffset + localTime );
}

int  
AVChunk::fixMe( int fix_lvl )
{
    int unfixed = 0;
    if ( _stream == 0 ) return 1;
    if ( _avfStreamOffset+length() > _stream->length() )
    {
        WARN( "avfStreamOffset+length > stream->length "
              <<_avfStreamOffset<<" "<<length()<<" "<<_stream->length() );
        if ( fix_lvl>0 && _stream->length() > _avfStreamOffset ) {
            setLength( _stream->length() - _avfStreamOffset );
            WARN( "fixed my length" );
        } else {
            unfixed++;
        }
    }
    return unfixed;
}

void 
AVChunk::print( std::ostream & o, int indent )
{
    string s( indent, ' ' );
    o << s << "AVChunk:"<<endl;
    o << s << "  inputFilename: "<< _stream->getFileName()<<endl;
    o << s << "  avfStreamOffset: "<<_avfStreamOffset<<endl;
    return;
}



