/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file invert.hh declares simple invert filter
 */

#if ! defined( PROPERTY_HH )
#define PROPERTY_HH

// std++
#include <iostream>
#include <string>
#include <vector>
#include <map>


namespace PIAVE {

    class PropertyNode;
    typedef std::vector< PropertyNode* > PropertyNodeList;
    typedef std::pair< std::string, std::string > Attribute;
    typedef std::map< std::string, std::string > AttrMap;


    class PropertyNode
    {
      public:
        PropertyNode( const std::string & n = std::string("") );
        PropertyNode( const PropertyNode & o );
        virtual ~PropertyNode();

        bool operator==( const PropertyNode & o ) const {
            return _name==o._name;
        }
        void operator=( const PropertyNode & o );

        /*   getter
         */
        std::string          getName()      const { return _name; }
        std::string          getContent()   const { return (*_content); }
        PropertyNode       * getDaughter( const std::string & n, int w=0, bool recurse = false );
        const PropertyNode * getDaughter( const std::string & n, int w=0, bool recurse = false ) const;
        PropertyNode       * getDaughter( unsigned int w=0 );
        const PropertyNode * getDaughter( unsigned int w=0 ) const;
        int                  getNDaughters() const { 
            return _daughters?_daughters->size():0;}

        bool getAttr( const std::string & a, std::string & v ) const;
        bool getAttr( const std::string & a, double & v ) const;
        bool getAttrRec( const std::string & a, std::string & v ) const;
        bool getAttrRec( const std::string & a, double & v ) const;

        /*   setter
         */
        void setName( const std::string & c );
        void setContent( const std::string & c );
        void addAttr( const std::string & n, const std::string & v );
        void addAttr( const std::string & n, const double & v );
        void addAttr( const std::string & n, const int & v );
        PropertyNode* addDaughter( const std::string & n );
        void addDaughter( PropertyNode* );

        void toXML( std::ostream & o, int indent = 2 ) const ;
        std::string toXML() const ;

        bool provides( PropertyNode * caps, bool recurse = true ) const;

      private:
        std::string         _name;
        std::string       * _content;
        AttrMap           * _attrMap;
        PropertyNodeList  * _daughters;
    };

}


#endif
