/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file avstream.cc implements avstream.hh
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif


// std++
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <string>
using std::string;
#include <cassert>

// this header
#include "streammgr.hh"

// PIAVE
#include "pluginmgr.hh"
#include "property.hh"
#include "exception.hh"
#include "avstream.hh"

using namespace PIAVE;


/*
 *  class StreamMgr 
 */

StreamMgr * PIAVE::StreamMgr::_singleton = 0;

StreamMgr::StreamMgr()
{
}


InAVStreamIFace * 
StreamMgr::getAVInput( const PropertyNode * p )
{
    ABORT("FIXME");
}


InAudioStreamIFace * 
StreamMgr::getAudioInput( const PropertyNode * p )
{
    ABORT("FIXME");
}



InAVFStream *
StreamMgr::getAVInput( const string& fileName )
{
    InAVFStream * instream = 0;

    if ( expandFileName( fileName ) == false ) {
        return 0;
    }

    if ( Inst()->_inAVStreams.find( fileName ) != Inst()->_inAVStreams.end() ) {
        INFO( "file already open, ok" );
        instream =  Inst()->_inAVStreams[ fileName ];
    } else {
        INFO( "file not yet open" );
        PropertyNode * stream_cap = new PIAVE::PropertyNode( "input" );
        stream_cap->addAttr( "filename", fileName );
        PluginBase * base = 0;
        const PluginMap & plugins = PluginMgr::getPluginMap();
        PluginMap::const_iterator iter;
        for( iter = plugins.begin();
             iter != plugins.end()  &&  !base;
             ++iter )
        {
            if ( iter->second->getType() == PLUGIN_INSTREAM ) {
                try {
                    base =  iter->second->newElement( stream_cap );
                    INFO( "Could open with "<<iter->first );
                } catch ( const stream_error & e ) {
                    INFO( "Couldn't open with "<<iter->first<<"  : "<<e.what() );
                    base = 0;
                }
            }
        }
	PDELETE( stream_cap );
        instream = dynamic_cast<PIAVE::InAVFStream*>( base );
        WARN_IF( instream==0 , "no plugin can handle "<<fileName );
    }

    if ( instream && Global::renderFmt.fps<0 ) {
        PropertyNode * is_props = instream->getProperties();
        assert( is_props );
        double fps=-1,w=-1,h=-1,a=-1;
        is_props->getAttrRec( "fps",    fps );
        is_props->getAttrRec( "width",  w );
        is_props->getAttrRec( "height", h );
        is_props->getAttrRec( "aspect", a );
        
        // FIXME  get aspect of screen //
        Global::renderFmt.fps     = (int) fps;    
        Global::renderFmt.width   = (int) w; 
        Global::renderFmt.height  = (int) h;
        Global::renderFmt.aspect  = a;

        INFO( is_props->toXML() );

    }

    return instream;
}


InAudioFStream * 
StreamMgr::getAudioInput( const std::string& fileName )
{
    ABORT("FIXME");
}



Stream * 
StreamMgr::getInput( const std::string& fileName )
{
    ABORT( "FIXME" );
    //  try to open as video, then also try audio
    //  the return value is crap... should be InFileStream, but that doesn't have
    //  getProperties() ... need a solution...
    return StreamMgr::getAVInput( fileName );
}



OutAVStreamIFace * 
StreamMgr::getOutAVStream( const PropertyNode * p )
{
    ABORT("FIXME");
}



OutAudioStreamIFace * 
StreamMgr::getOutAudioStream( const PropertyNode * p )
{
    ABORT("FIXME");
}


bool
StreamMgr::expandFileName( string fileName )
{
    int fd = open( fileName.c_str(), O_RDONLY | O_NONBLOCK );
    if ( fd >=0 ) {
        close( fd );
        return true;
    }

    if ( fileName[0] != '/' ) {
        for( std::vector< string >::const_iterator path = Inst()->_pathList.begin();
             path != Inst()->_pathList.end();
             ++path )
        {
            string fullFileName = (*path) + "/" + fileName;
            int fd = open( fullFileName.c_str(), O_RDONLY | O_NONBLOCK );
            if ( fd >=0 ) {
                close( fd );
                return true;
            }
        }
    }    

    INFO( "could't expand filename " << fileName );
    return false;
}

