/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** 
 * \file frame.cc implements frame.hh
 *
 * based on:
 *
 * frame.cc -- utilities for process digital video frames
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif

// this header
#include "videoformat.hh"

// piave
#include "piave_base.hh"

// std c++
#include <string>
using std::string;
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

using namespace PIAVE;


const FOURCC FOURCC::NONE( (uint32_t)0x00000000 );
const FOURCC FOURCC::RGB ( (uint32_t)0x00000003 );
const FOURCC FOURCC::YUY2( "YUY2" );


const PixFmt PixFmt::NONE    ( FOURCC::NONE, flat  );
const PixFmt PixFmt::RGB     ( FOURCC::RGB,  flat  );
const PixFmt PixFmt::RGB_A   ( FOURCC::RGB,  alpha );
const PixFmt PixFmt::YUY2    ( FOURCC::YUY2, flat  );
const PixFmt PixFmt::YUY2_A  ( FOURCC::YUY2, alpha );

std::string 
FOURCC::toString() const
{
    if ( (*this)==RGB ) {
        return string( "RGB" );
    } else {
        const char * c = reinterpret_cast<const char *>( &fourcc );
        return string( c, 4 );
    }
    return string("ERR");
}

std::ostream& PIAVE::operator<<( std::ostream&o, const FOURCC& f )
{
    o << "\""<< f.toString() << "\"";
    return o;
}

std::ostream& PIAVE::operator<<( std::ostream&o, const PixFmt& f )
{
    o << "fourcc: " << f.getFOURCC().toString();
    switch ( f.getAlpha() ) {
      case PixFmt::alpha: o << ":alpha"; break;
      case PixFmt::flat: o << ":flat"; break;
      default: o << " unknown";
    }
    return o;
}
