/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, Rolf.Dubitzky@gmx.net
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


// STDC
#include <unistd.h>
#include <getopt.h>

// STDC++
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
using std::string;

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif

// For translations

#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  if ! defined(textdomain)
#    define textdomain(String) (String)
#  endif
#  if ! defined(gettext)
#    define gettext(String) (String)
#  endif
#  if ! defined(dgettext)
#    define dgettext(Domain,Message) (Message)
#  endif
#  if ! defined(dcgettext)
#    define dcgettext(Domain,Message,Type) (Message)
#  endif
#  if ! defined(bindtextdomain)
#    define bindtextdomain(Domain,Directory) (Domain)
#  endif
#  if ! defined(_)
#    define _(String) (String)
#  endif
#  if ! defined(N_)
#    define N_(String) (String)
#  endif
#endif

// PIAVE
#include "libpiave/piave_base.hh"
#include "libpiave/kdenserver.hh"
#include "libpiave/pluginmgr.hh"

void
usage()
{
    cout << _(" -h            this ") << endl
         << _(" -p <n>        portnumber to listen for kdenlive; default: 6100") << endl
         << _(" -e            don't listen for SDL events (debugging)   ") << endl
//         << _(" -a            full size; don't use half size preview") << endl
         << _(" -v            verbose; print WARNings") << endl
         << _(" -d            debug; print INFOs") << endl << endl;

    /*
     *   init piave
     */
    PIAVE::Init();
    PIAVE::PluginMgr::openDLLsFromDefaultPath();

    /*
     *   create a dummy server to get list of all available VEML commands
     */
    PIAVE::KdenLiveServer kden( 0 );
    kden.printAllVEML( cout );

    return;
}

int
main( int argc,  char * const * argv )
{

    unsigned short  portnumber = 6100;
    bool listenForSDLEvents = true;

    PIAVE::Global::verbose = false;
    PIAVE::Global::quiet = true;

    int c;
    opterr = 0;
    while ((c = getopt (argc, argv, "eahvdp:f:")) != -1) {
        switch (c)
        {
          case 'p': 
          {
              char * endptr;
              long int p = strtol( optarg, &endptr, 10);
              if ( (*endptr) == '\0' ) {
                  portnumber = static_cast<unsigned short>( p );
              }
          } break;
          case 'f': 
          {
              string format = optarg;
              if ( format == "rgb" ) {
//                  sdl_format = PIAVE::OutSDLStream::rgb_surface;
              } else if ( format == "yuv" ) {
//                  sdl_format = PIAVE::OutSDLStream::yuv_overlay;
              } else {
                  cerr << "Warning: unknown format \""<<format<<"\""<<endl;
              }
          }
          case 'v':
              PIAVE::Global::quiet = false;
              break;
          case 'a':
//              PIAVE::Global::preview = false;
              break;
          case 'd':
              PIAVE::Global::quiet = false;
              PIAVE::Global::verbose = true;
              break;
          case 'e':
              listenForSDLEvents = false;
              break;
          case 'h':
              usage();
              return 0;
          default:
              usage();
              return 1;
        }
    }

    /*
     *   init piave
     */
    PIAVE::Init();
    PIAVE::PluginMgr::openDLLsFromDefaultPath();

    /*
     *   start server
     */
    PIAVE::KdenLiveServer kden( portnumber );
//    kden.setSDLformat( sdl_format );
    kden._listenForSDLEvents = listenForSDLEvents;
    kden.execute();

    return 0;
}
