/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/** 
 * \file filter.cc implements filter.hh
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif

#include "alphablend.hh"
#include "libpiave/property.hh"

using namespace PIAVE;


PIAVE_PLUGIN( AlphaBlendPlugin );



/**
 *
 *  class AlphaBlend
 *
 */

AlphaBlendOp::AlphaBlendOp() : 
  BinaryOp( "AlphaBlendOp" ), 
  _amplitude( "amplitude" , 0.5 ) 
{
    _amplitude.setAbout("0.0: track A; 1.0 track B opaque" );
    _amplitude.setIsDynamic();
    addParameter( &_amplitude );
};

void
AlphaBlendOp::renderFrame( Frame & dest, Frame & src_a, Frame & src_b, Time t ) 
{
    ABORT( "FIXME" );
#if 0
    Time localTime = t - start();

    VideoBuffer * sa_vb = src_a.makeVBuf();

    uint8_t * sa = sa_vb->getPixels();
    uint8_t * sb = sb_vb->getPixels();

    int sa_step = gdk_pixbuf_get_n_channels( sa_vb->getPixBuf() );
    int sb_step = gdk_pixbuf_get_n_channels( sb_vb->getPixBuf() );

    double rb = interpolateVal( &AlphaBlendOpP::amplitude, localTime );
    double ra = 1.0 - rb;

    // these 'if's is pretty ugly but I don't know how to be 
    // elegant and avoid double looping or double
    // dereferencing
    if ( sa_vb->hasAlpha() ) {
        dest = src_a;
        dest.makeWriteableVBuf();
        VideoBuffer * dest_vb = dest.getVBuf();
        uint8_t * d = dest_vb->getPixels();
        int d_step  = gdk_pixbuf_get_n_channels( dest_vb->getPixBuf() );
        if ( sb_vb->hasAlpha() ) {
            for ( unsigned int i = 0; i<Global::render_size; i++ ) {
                (*(d+3)) = uint8_t( ra * (*(sa+3)) + rb * (*(sb+3)) );
                if ( *(d+3) > 2 ) {
                    (*(d+0)) = uint8_t( ra * (*(sa+0)) + rb * (*(sb+0)) );
                    (*(d+1)) = uint8_t( ra * (*(sa+1)) + rb * (*(sb+1)) );
                    (*(d+2)) = uint8_t( ra * (*(sa+2)) + rb * (*(sb+2)) );
                }
                d  += d_step;
                sa += sa_step;
                sb += sb_step;
            }
        } else {
            for ( unsigned int i = 0; i<Global::render_size; i++ ) {
                (*(d+3)) = uint8_t( ra * (*(sa+3)) + rb * 255 );
                if ( *(d+3) > 2 ) {
                    (*(d+0)) = uint8_t( ra * (*(sa+0)) + rb * (*(sb+0)) );
                    (*(d+1)) = uint8_t( ra * (*(sa+1)) + rb * (*(sb+1)) );
                    (*(d+2)) = uint8_t( ra * (*(sa+2)) + rb * (*(sb+2)) );
                }
                d  += d_step;
                sa += sa_step;
                sb += sb_step;
            }
        }
    } else {
        if ( sb_vb->hasAlpha() ) {
            dest = src_b;
            dest.makeWriteableVBuf();
            VideoBuffer * dest_vb = dest.getVBuf();
            int d_step  = gdk_pixbuf_get_n_channels( dest_vb->getPixBuf() );
            uint8_t * d = dest_vb->getPixels();
            for ( unsigned int i = 0; i<Global::render_size; i++ ) {
                (*(d+3)) = uint8_t( ra * 255 + rb * (*(sb+3)) );
                if ( *(d+3) > 2 ) {
                    (*(d+0)) = uint8_t( ra * (*(sa+0)) + rb * (*(sb+0)) );
                    (*(d+1)) = uint8_t( ra * (*(sa+1)) + rb * (*(sb+1)) );
                    (*(d+2)) = uint8_t( ra * (*(sa+2)) + rb * (*(sb+2)) );
                }
                d  += d_step;
                sa += sa_step;
                sb += sb_step;
            }
        } else {
            dest = src_a;
            dest.makeWriteableVBuf();
            VideoBuffer * dest_vb = dest.getVBuf();
            int d_step  = gdk_pixbuf_get_n_channels( dest_vb->getPixBuf() );
            uint8_t * d = dest_vb->getPixels();
            for ( unsigned int i = 0; i<Global::render_size; i++ ) {
                (*(d+0)) = uint8_t( ra * (*(sa+0)) + rb * (*(sb+0)) );
                (*(d+1)) = uint8_t( ra * (*(sa+1)) + rb * (*(sb+1)) );
                (*(d+2)) = uint8_t( ra * (*(sa+2)) + rb * (*(sb+2)) );
                d  += d_step;
                sa += sa_step;
                sb += sb_step;
            }
        }
    }
#endif
}



/*
 *   Plugin handler for invert filter
 *
 *
 */
AlphaBlendPlugin::AlphaBlendPlugin()
 : Plugin( "alphablend", PLUGIN_VIDEO_OP ) 
{
    PropertyNode * p;
    PropertyNode * caps = new PropertyNode( "effect" );
    caps->addAttr( "name", getName() );

    p = caps->addDaughter( "input" );
    p->addAttr( "name", "A" );
    p->addAttr( "video", "yes" );
    p->addAttr( "audio", "yes" );

    p = caps->addDaughter( "input" );
    p->addAttr( "name", "B" );
    p->addAttr( "video", "yes" );
    p->addAttr( "audio", "yes" );

    p = caps->addDaughter( "parameter" );
    p->addAttr( "name", "fade" );
    p->addAttr( "type", "double" );
    p->addAttr( "min", "0.0" );
    p->addAttr( "max", "1.0" );
    p->setContent( "Controls the transparency of video B. "
                   "At 0.0, video A is completely on screen. "
                   "At 1.0, video B is completely visible.\n" );

    PropertyNode * preset = caps->addDaughter( "preset" );
    preset->addAttr( "name", "crossfade" );
    PropertyNode * fade = preset->addDaughter( "fade" );
    p = fade->addDaughter( "keyframe" );
    p->addAttr( "time", "0.0" );
    p->addAttr( "value", "0.0" );
    p = fade->addDaughter( "keyframe" );
    p->addAttr( "time", "1.0" );
    p->addAttr( "value", "1.0" );

    preset = caps->addDaughter( "preset" );
    preset->addAttr( "name", "overlay" );
    fade = preset->addDaughter( "fade" );
    p = fade->addDaughter( "keyframe" );
    p->addAttr( "time", "0.0" );
    p->addAttr( "value", "0.5" );
    p = fade->addDaughter( "keyframe" );
    p->addAttr( "time", "1.0" );
    p->addAttr( "value", "0.5" );

    preset = caps->addDaughter( "preset" );
    preset->addAttr( "name", "inandout" );
    fade = preset->addDaughter( "fade" );
    p = fade->addDaughter( "keyframe" );
    p->addAttr( "time", "0.0" );
    p->addAttr( "value", "0.0" );
    p = fade->addDaughter( "keyframe" );
    p->addAttr( "time", "0.2" );
    p->addAttr( "value", "1.0" );
    p = fade->addDaughter( "keyframe" );
    p->addAttr( "time", "0.8" );
    p->addAttr( "value", "1.0" );
    p = fade->addDaughter( "keyframe" );
    p->addAttr( "time", "1.0" );
    p->addAttr( "value", "0.0" );

    PropertyNode * about = caps->addDaughter( "about" );
    about->setContent( "This binary operator can be used to overlay a still images,\n"
                       "fade in text, cross fade transitions, etc, etc...\n" );

    setCapabilities( caps );
};

PluginBase * 
AlphaBlendPlugin::newElement( const PropertyNode * p )
{
    // FIXME check initialization 
    return new AlphaBlendOp();
}
