/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file image.hh implements simpe still and dynamic color effects
 */
#if ! defined( IMAGE_HH )
#define IMAGE_HH

// PIAVE
#include "libpiave/storybase.hh"
#include "libpiave/plugin.hh"

namespace PIAVE {

    class ColorStillPlugin : public Plugin
    {
      public:
        ColorStillPlugin();
        virtual PluginBase * newElement( const PropertyNode * p = 0 );
    };

//     class StillImagePlugin : public Plugin
//     {
//       public:
//         ImagePlugin();
//         virtual PluginBase * newElement( const PropertyNode * p = 0 );
//     };

    /**
     * represents a simple uni color image 
     * \todo port to pixbuf and support alpha channel
     */
    class ColorStill : public MediaElement
    {
      public:
        ColorStill( const std::string& name = std::string() );
        virtual ~ColorStill();

        virtual Frame getFrame( Time t );
        virtual PropertyNode * getProperties( PropertyNode * p  = 0 ) const;

        virtual void print( std::ostream & o, int indent );

        bool useAlpha() const { return _useAlpha; };
        bool setUseAlpha( bool u = true ) { _useAlpha = u; return _useAlpha; };

      public: // private:
        void _init();

        bool _initialized;
        bool _useAlpha;
        Color _color;        // the color
        VideoBuffer * _vb;   // the pixels

    };



    /**
     *    reimplement based in SDL to reduce dependecies
     *
     * represents time independent bitmap image 
     * based on gdk-pixbuf 
     * all formats supported by gdk-pixbuf are ok.
     * alpha channel is supported
     *
     * \todo get rid of _pixbuf -> load directly to frame
     *
     */
//     class StillImage : public MediaElement
//     {
//       public:
//         StillImage( const std::string& filename = std::string(), int x_off = 0, int y_off = 0 );
//         virtual ~StillImage();
        
//         virtual Frame getFrame( Time t );

//         virtual void print( std::ostream & o, int indent );

//       private: 
//         void _init();

//       public: // private:
//         int _x_off, _y_off;
//         /// buffer
//         VideoBuffer  * _vb;
//     };

}

#endif
