/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/** 
 * \file filter.cc implements filter.hh
 */

// config
#if defined( HAVE_CONFIG_H )
#    include "config.h"
#endif

#include "invert.hh"
#include "libpiave/property.hh"
#include "libpiave/bufferpool.hh"

using namespace PIAVE;

/*
 *   Plugin handler for invert filter
 *
 *
 */
PIAVE_PLUGIN( InvertFilterPlugin );

InvertFilterPlugin::InvertFilterPlugin()
 : Plugin( "InvertFilter", PLUGIN_VIDEO_OP ) 
{
    PropertyNode * caps = new PropertyNode( "effect" );
    caps->addAttr( "name", getName() );

    PropertyNode * p;
    p = caps->addDaughter( "input" );
    p->addAttr( "input", "A" );
    p->addAttr( "video", "yes" );

    p = caps->addDaughter( "about" );
    p->setContent( "This filter is a simple color inversion, "
                   "no parameters.\n" );

    setCapabilities( caps );
};

PluginBase * 
InvertFilterPlugin::newElement( const PropertyNode * p )
{
    return new InvertFilter( "InvertFilter" );
}


/*
 *   \class InvertFilter
 *
 *
 */
void
InvertFilter::renderFrame( Frame & f, Time t )
{
    if ( f.getVBuf()->isLocked() )
        f.makeWriteableVBuf();

    uint8_t * p = f.getVBuf()->getPixels();
    uint8_t * end = p + f.getVBuf()->getSizeBytes();
    while ( p != end ) {
        *p = 0xff - *p;
        p++;
    }

    return;
}

