/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file textmaster defines a text operator based in FreeType
 * \see UnaryOp
 */

#if ! defined( TEXTMASTER_HH )
#define TEXTMASTER_HH


//PIAVE 
#include "libpiave/operatorbase.hh"

// freetype2
#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_GLYPH_H

namespace PIAVE 
{

    /*
     *  \class FreeType is a simple wrapper around freetype
     */
    struct FreeType 
    {
        static void init();
        static bool is_initialized;
        static FT_Library  library;
    };

    class TextMaster  : public UnaryOp
    {
      public:
        enum HorizAlign { left, hcenter, right };
        enum VertAlign { top, vcenter, bottom };

      public:
        TextMaster();
     
        virtual void renderFrame( Frame & f, Time t );

        virtual PropertyNode * getProperties( PropertyNode * p=0 ) const;

      int fixMe( int fix_lvl );

      private:
        /// prerender a glyph from the text
        void initGlyphs();
        /// draw a glyph onto a video Frame
        void renderGlyphBitmapToFrame( Frame & f, const FT_Bitmap&, 
                                       const FT_Vector& pos, const Color color );

      public:  // private: \todo cleanup
        /// true after glyphes have been initialized
        bool             _glyphs_initialized;
        /// holds the text ; some day also multiline
        OpParString      _text;
        /// filename of the .ttf
        OpParString      _fontName;
        // alignement
        HorizAlign       _horizAlign;
        VertAlign        _vertAlign;
        OpParDouble      _size;
        /// offset for a shadow  (0,0)= no shadow
        Point _shadow_offset;
        OpParBox         _position;
        OpParColor       _color;


        /*
         *   FreeType font render caches
         */

        /// caches the glyphs
        std::vector< FT_Glyph >   _glyphs;
        /// position of glyphs
        std::vector< FT_Vector >  _glyphPos;
        // 
        FT_Face       _face; 
   
    };


    class TextMasterPlugin : public Plugin
    {
      public:
        TextMasterPlugin();
        virtual PluginBase * newElement( const PropertyNode *p = 0 );
    };

};

#endif
