/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file ossstream.hh
 */

#if ! defined( OSSSTREAM_HH )
#define OSSSTREAM_HH

// stdc
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

// stdc++
#include <string>

// PIAVE
#include "libpiave/piave_base.hh"
#include "libpiave/avstream.hh"
#include "libpiave/exception.hh"

// SDL
#include <SDL/SDL.h>
#include <SDL/SDL_syswm.h>

namespace PIAVE {

    class Frame;

    class OssPlugin : public Plugin
    {
      public:
        OssPlugin();
        virtual PluginBase * newElement( const PropertyNode * p = 0 );
    };


    /** 
     * use oss to play audio
     */ 
    class OutOSSStream : public OutAudioStreamIFace
    {
      public:
        OutOSSStream();
        ~OutOSSStream();
        virtual void putFrame( Frame & f );

        virtual PropertyNode * getProperties( PropertyNode * p ) const;

      private:
        void init( Frame & f ) throw(exception);

      private:
        int            _fd;
        int            _nChannels;
        int            _frequency;
        std::string    _deviceName;
        
    };

};

#endif
