/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file rawdvstream.hh use libdv for raw DV IO
 */

#if ! defined( LIBDV_CODEC_HH )
#define LIBDV_CODEC_HH

// PIAVE
#include "libpiave/piave_base.hh"
#include "libpiave/avstream.hh"

// other libs
#include <libdv/dv.h>
#include <libdv/dv_types.h>


namespace PIAVE {

    /*
     *    some constants
     */
    struct LibDV 
    {
        static const ssize_t       PAL_FRAMESIZE  = 144000;   
        static const ssize_t       NTSC_FRAMESIZE = 120000;   
        static const double        PAL_FPS        = 25;       
        static const unsigned int  PAL_WIDTH      = 720;      
        static const unsigned int  PAL_HEIGHT     = 576;      
        static const double        NTSC_FPS       = 29.97;    
        static const unsigned int  NTSC_WIDTH     = 720;      
        static const unsigned int  NTSC_HEIGHT    = 480;      
        static const double        ASPECT_4x3     = 4.0/3.0;  
        static const double        ASPECT_16x9    = 16.0/9.0; 

        /* that's more than enough dummy space  */
        static uint8_t             tmp_buffer[ 720 * 576 * 3 ];
    };


    /*
     *    plugin wrapper for raw DV codec
     */
    class LibDVDecoderPlugin : public Plugin
    {
      public:
        LibDVDecoderPlugin();
        virtual PluginBase * newElement( const PropertyNode * p = 0 );
    };

    class LibDVEncoderPlugin : public Plugin
    {
      public:
        LibDVEncoderPlugin();
        virtual PluginBase * newElement( const PropertyNode * p = 0 );
    };

    /**
     *  Class InRawDVStream
     *  handles raw dv streams as produced by libdv, i.e. dvgrab
     *  \todo check VideoInfo and compare to PIAVE::Global
     */

    class LibDVDecoder : public AVDecoderIFace
    {
      public:
        LibDVDecoder();

        virtual void decodeMetaData( Frame & f, PropertyNode * p );
        virtual void decodeVideo( Frame & f, uint8_t * dest = 0 );
        virtual void decodeAudio( Frame & f, uint8_t * dest = 0 );

        virtual PropertyNode * getProperties( PropertyNode * p) const;
//        virtual PropertyNode * getFrameProperties( PropertyNode * ) const;

        virtual bool init( uint8_t * data = 0 ) throw (exception);
        dv_decoder_t * get_dv_decoder() const { return _dv_decoder; }
        
        int  getFrameSize() const { return _frameSize; }
        bool isNewRecording( Frame * f ) const;


      private:
        dv_decoder_t     * _dv_decoder;
        dv_system_t        _dv_video_system;
        int                _frameSize;

        int                _pitches[3];
        uint8_t          * _pixels[3];

    };

    class LibDVEncoder : public AVEncoderIFace
    {
      public:
        LibDVEncoder();
        virtual void encodeVideo( Frame & f, uint8_t * dest = 0 ) ;
        virtual void encodeAudio( Frame & f, uint8_t * dest = 0 ) ;
        virtual PropertyNode * getProperties( PropertyNode * p ) const;
        int  getFrameSize() const { return _frameSize; }

        virtual bool init( uint8_t * data = 0 ) throw (exception);
                
      private:
        dv_encoder_t *     _dv_encoder;
        dv_system_t        _dv_video_system;
        int                _frameSize;

        int                _pitches[3];
        uint8_t          * _pixels[3];

    };


}

#endif
