/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file rawdvstream.hh use libdv for raw DV IO
 */

#if ! defined( RAWDVSTREAM_HH )
#define RAWDVSTREAM_HH

// PIAVE
#include "libpiave/plugin.hh"
#include "libpiave/avstream.hh"


namespace PIAVE {

    class LibDVDecoder;
    class LibDVEncoder;
    class PropertyNode;
    class Frame;

    /*
     *    plugin wrapper for raw DV stream
     */
    class InRawDVStreamPlugin : public Plugin
    {
      public:
        InRawDVStreamPlugin();
        virtual PluginBase * newElement( const PropertyNode * p = 0 );
    };

    class OutRawDVStreamPlugin : public Plugin
    {
      public:
        OutRawDVStreamPlugin();
        virtual PluginBase * newElement( const PropertyNode * p = 0 );
    };


    /*
     *  Class InRawDVStream
     *  handles raw dv streams as produced by libdv, i.e. dvgrab
     *  \todo check VideoInfo and compare to PIAVE::Global
     */
    class InRawDVStream : public InAVFStream
    {
      public:
        InRawDVStream( const std::string& fileName ) throw(exception);
        virtual Frame getFrame( Time t );

        virtual void decodeMetaData( Frame & f );
        virtual void decodeVideo( Frame & f, uint8_t * dest = 0 );
        virtual void decodeAudio( Frame & f, uint8_t * dest = 0 );

        virtual PropertyNode * getProperties( PropertyNode * p ) const;

      private:
        LibDVDecoder  * _dvDecoder;
        int _frameSize;
    };

    class OutRawDVStream : public OutAVFStream
    {
      public:
        OutRawDVStream( const std::string& fileName );
        virtual ~OutRawDVStream() { INFO( "~OutRawDVStream" ); }

        virtual void putFrame( Frame & f );

        virtual PropertyNode * getProperties( PropertyNode * p=0 ) const;

      private:
        LibDVEncoder  * _dvEncoder;
        int _frameSize;
    };


}

#endif
