/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file sdloutstream.hh   display images with SDL
 */

#if ! defined( SDLOUTSTREAM_HH )
#define SDLOUTSTREAM_HH

// PIAVE
#include "libpiave/piave_base.hh"
#include "libpiave/avstream.hh"

// SDL
#include <SDL/SDL.h>
#include <SDL/SDL_syswm.h>

namespace PIAVE {

    class PropertyNode;

    class SDLStreamPlugin : public Plugin
    {
      public:
        SDLStreamPlugin();
        virtual PluginBase * newElement( const PropertyNode * p = 0 );
    };

    /**
     * class OutSDLStream
     * use SDL to display single frames
     */
    class OutSDLStream : public OutAVStreamIFace
    {
      public:
        enum fmt_e { no_fmt, yuv_overlay, rgb_surface };
        
        OutSDLStream();
	~OutSDLStream();
        virtual void putFrame( Frame & f );

        virtual PropertyNode * getProperties( PropertyNode * p=0 ) const;

        void init();
        const SDL_SysWMinfo & getSysWMinfo();

        void setIgnoreInputEvents();

      private:
        bool                   _SDL_init_OK;
        SDL_Surface          * _screen;
        SDL_Surface          * _image;
        SDL_SysWMinfo          _sysWMinfo;
        const SDL_VideoInfo  * _videoInfo;
        SDL_Overlay          * _overlay;
        SDL_Rect               _rect;
        fmt_e                  _fmt;

        friend class SDLStreamPlugin;
    };

}

#endif
