/*
 * PIAVE - PIAVE Is A Video Editor
 *
 * Copyright (C) 2002 Rolf Dubitzky, rolf@dubitzky.de
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file vorbisstream.hh uses the simple vorbisfile API to access
 * plain Ogg/Vorbis files.
 * Supporets plain Ogg/Vorbis I streams, i.e. meta-header less, unmultiplexed
 * ogg files with Vorbis I audio packets. 
 * This is a pre-stage of an implementation of seperatre
 * Ogg container and Vorbis codec plugins
 */

#if ! defined( VORBISFILE_HH )
#define VORBISFILE_HH

// stdc
#include <signal.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

// stdc++
#include <string>

// vorbis
#include "vorbis/codec.h"
#include "vorbis/vorbisfile.h"

// PIAVE
#include "libpiave/piave_base.hh"
#include "libpiave/avstream.hh"
#include "libpiave/exception.hh"
#include "libpiave/ringbuffer.hh"


namespace PIAVE {

    class Frame;

    class VorbisfilePlugin : public Plugin
    {
      public:
        VorbisfilePlugin();
        virtual PluginBase * newElement( const PropertyNode * p = 0 );
    };


    /** 
     * use oss to play audio
     */ 
    class InVorbisfile : public InAVFStream
    {
      public:
        InVorbisfile( const std::string& fileName ) throw(exception);

        ~InVorbisfile();
        virtual Frame getFrame( Time t );

        virtual PropertyNode * getProperties( PropertyNode * p ) const;

        virtual void decodeMetaData( Frame & f );
        virtual void decodeVideo( Frame & f, uint8_t * dest = 0 );
        virtual void decodeAudio( Frame & f, uint8_t * dest = 0 );

      private:
        void refillBuffer();

      private:
        FILE           * _file; /// should go to InFileStream
        OggVorbis_File   _vf;
        vorbis_info    * _vi;
        vorbis_comment * _vc;
        int              _current_section;
        int              _nChannels;
        int              _frequency;

        /** data is read from the file to this ring buffer the buffer can
         *  contain data for the next maxSec. In case it holds less
         *  then minSec seconds, it will be refilled
         */
        ValRingBuffer<int16_t> * _readBuffer;
        double                   _minSec;
        double                   _maxSec;

    };

};

#endif
